@@ -66,7 +66,7 @@ SEXP HazardBs23R(SEXP x, SEXP nph, SEXP timecat, SEXP fixobs, SEXP param, SEXP p
   double *MyParam = (double *)R_alloc((nbase+1),sizeof(double));
   double *TempD = (double *)R_alloc(Cst1,sizeof(double));
 
-  double (*Fpt)(double, double*, double*, double*, double*, int);
+  double (*Fpt)(double, double*, double*, double*, double*);
   if (Deg==2){
     Fpt = &Spline2;
   }
@@ -85,7 +85,7 @@ SEXP HazardBs23R(SEXP x, SEXP nph, SEXP timecat, SEXP fixobs, SEXP param, SEXP p
     }
     CumVecSpl[0] = 0;
     for (i=0; i<lk; i++){
-      tempV = IntSpline23((*Fpt), TotKPos[i-1], TotKPos[i], &TotK[i], &MatK[Cst2*i], TempD, &MyParam[i], N, lW, lleg, Cst1);
+      tempV = IntSpline23((*Fpt), TotKPos[i-1], TotKPos[i], &TotK[i], &MatK[Cst2*i], TempD, &MyParam[i], N, lW, lleg);
       CumVecSpl[i+1] = CumVecSpl[i] + tempV;
     }
     for (z=0; z<lx; z++){
@@ -96,8 +96,8 @@ SEXP HazardBs23R(SEXP x, SEXP nph, SEXP timecat, SEXP fixobs, SEXP param, SEXP p
       }
       tcz = TimeCat[z];
       tempL = CumVecSpl[tcz];
-      tempL += IntSpline23((*Fpt), TotKPos[tcz-1], X[z], &TotK[tcz], &MatK[Cst2*tcz], TempD, &MyParam[tcz], N, lW, lleg, Cst1);
-      tempH = (*Fpt)(X[z], &TotK[tcz], &MatK[Cst2*tcz], TempD, &MyParam[tcz], Cst1);
+      tempL += IntSpline23((*Fpt), TotKPos[tcz-1], X[z], &TotK[tcz], &MatK[Cst2*tcz], TempD, &MyParam[tcz], N, lW, lleg);
+      tempH = (*Fpt)(X[z], &TotK[tcz], &MatK[Cst2*tcz], TempD, &MyParam[tcz]);
       Total += tempL + tempH + tempF;
       LogHaz[z] = tempH + tempF;
       HazCum[z] = tempL*exp(tempF);
@@ -121,10 +121,10 @@ SEXP HazardBs23R(SEXP x, SEXP nph, SEXP timecat, SEXP fixobs, SEXP param, SEXP p
       tcz = TimeCat[z];
       tempL = 0;
       for (i=0; i<tcz; i++){
-        tempL += IntSpline23((*Fpt), TotKPos[i-1], TotKPos[i], &TotK[i], &MatK[Cst2*i], TempD, &MyParam[i], N, lW, lleg, Cst1);
+        tempL += IntSpline23((*Fpt), TotKPos[i-1], TotKPos[i], &TotK[i], &MatK[Cst2*i], TempD, &MyParam[i], N, lW, lleg);
       }
-      tempL += IntSpline23((*Fpt), TotKPos[tcz-1], X[z], &TotK[tcz], &MatK[Cst2*tcz], TempD, &MyParam[tcz], N, lW, lleg, Cst1);
-      tempH = (*Fpt)(X[z], &TotK[tcz], &MatK[Cst2*tcz], TempD, &MyParam[tcz], Cst1);
+      tempL += IntSpline23((*Fpt), TotKPos[tcz-1], X[z], &TotK[tcz], &MatK[Cst2*tcz], TempD, &MyParam[tcz], N, lW, lleg);
+      tempH = (*Fpt)(X[z], &TotK[tcz], &MatK[Cst2*tcz], TempD, &MyParam[tcz]);
       Total += tempL + tempH + tempF;
       LogHaz[z] = tempH + tempF;
       HazCum[z] = tempL*exp(tempF);
